package com.cds.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;

import com.cds.model.Article;
import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;

public class UserArticle {
	@Autowired
	private MongoTemplate mongoTemplate;
	
	public UserArticle(MongoTemplate template){
		this.mongoTemplate = template;
	}
	 
	/**
	 * get all the Keys that are associated with this Document ID
	 * @param docid 
	 * @return return List of Keys 
	 */
	public Set<String> getArticleKeys(String docid){
		
		DBCursor cursor;
		DBObject object ;
		DBCollection collection = mongoTemplate.getCollection("article");
		BasicDBObject query  = new BasicDBObject("docid", docid);
		cursor = collection.find(query);
		if(cursor != null){ 
			object = cursor.next();
			return object.keySet();
		}else{
			return null;
		}
	}	   
	/***
	 * method to get all the attribute keys for anonymous user
	 * @param docid
	 * @return returns list of keys which are visible to the user
	 */
	public List<String> getIncludedKeysForAnonymousUser(String docid){
		List<String> list = new ArrayList<String>();
		
		/** check to see if the root level permissions is public for this anonymous user, if not do not show anything**/
		Query query = new Query();
		Criteria criteria = Criteria.where("permissions").is("public").and("docid").is(docid);
		query.addCriteria(criteria);
		Article rootArticle = mongoTemplate.findOne(query,Article.class);
		if(rootArticle == null ){
			return list; // return empty list as article level permissions is not public for anonymous user.
		}
		
		Set<String> set = getArticleKeys(docid);
		for (String string : set) {
			/** build a query to only show public content as this is anonymous user*/ 
			
			Query query1 = new Query();
			Criteria criteria1 = Criteria.where(string+".permission").is("public").and("docid").is(docid);
			query1.addCriteria(criteria1);
			List<Article> articles = mongoTemplate.find(query1,Article.class);
			if(articles!=null && articles.size()>0){
				list.add(string);
			}
		}
		
		/** for now there is not security specified for docid and locale, just show them **/
		list.add("docid");list.add("locale");
		
		return list;
	}
	
	/**
	 * this method gets all the valid Keys associated with the User permission, note we need to check for public content keys also 
	 * @param permission
	 * @param docid
	 * @return
	 */
	public  List<String> getIncludedKeys(String permission,String docid){
		List<String> list = new ArrayList<String>();
		
		/** check to see if the root level permissions is either public or user specific permission, if not do not show anything**/
		Query query1 = new Query();
		Criteria criteria1 = new Criteria().orOperator(Criteria.where("permissions").is(permission).and("docid").is(docid),  
				  Criteria.where("permissions").is("public").and("docid").is(docid));
		query1.addCriteria(criteria1);
		Article rootArticle = mongoTemplate.findOne(query1,Article.class); 
		if(rootArticle == null ){
			return list; // return empty list as article level permissions is not public or not visible to this user permission.
		}

		Set<String> set = getArticleKeys(docid);
		for (String string : set) {
			Query query = new Query();
			Criteria criteria = new Criteria().orOperator(Criteria.where(string+".permission").is(permission).and("docid").is(docid),  
                    									  Criteria.where(string+".permission").is("public").and("docid").is(docid));
			query.addCriteria(criteria);
			List<Article> articles = mongoTemplate.find(query,Article.class);
			if(articles!=null && articles.size()>0){
				list.add(string);
			}
		}
		list.add("docid");list.add("locale");
		return list;
	}
	

	
	/**
	 * This method return the Permission associated with this USER
	 * @param user
	 * @return return what permission at the TOP level user has. this method further needs to expand hierarchical way
	 */
	public   String getUserPermissions(String user){
		
		if(user == null ) return null;
		
		DBCursor cursor;
		DBCollection collection = mongoTemplate.getCollection("user");
		System.out.println("getUserPermissions() : "+user);
		BasicDBObject query = new BasicDBObject("user", user.trim());
		BasicDBObject keys  = new BasicDBObject("permissions.tag", 1).append("_id", 0);
	    cursor = collection.find(query,keys);		
		String permission = "";
		if(cursor != null){
			try {
			DBObject o = cursor.next();
 			String jsonString = o.toString();
 			System.out.println(jsonString);
			
				JSONObject json = (JSONObject)new JSONParser().parse(jsonString);
				json = (JSONObject)new JSONParser().parse(json.get("permissions").toString());
				permission = (String)json.get("tag");
			} catch (Exception e) {
 				 permission = null;
			}
			
		}
		return permission;
	}	
	
	
}
